from zempy.zosapi.core.enum_base import ZosEnumBase


class Beam(ZosEnumBase):
    Reference = 0
    Configuration_1 = 1
    Configuration_2 = 2
    Configuration_3 = 3
    Configuration_4 = 4
    Configuration_5 = 5
    Configuration_6 = 6
    Configuration_7 = 7
    Configuration_8 = 8
    Configuration_9 = 9
    Configuration_10 = 10
    Configuration_11 = 11
    Configuration_12 = 12
    Configuration_13 = 13
    Configuration_14 = 14
    Configuration_15 = 15
    Configuration_16 = 16
    Configuration_17 = 17
    Configuration_18 = 18
    Configuration_19 = 19
    Configuration_20 = 20

Beam._NATIVE_PATH = "ZOSAPI.Analysis.Beam"
Beam._ALIASES_EXTRA = {}

__all__ = ["Beam"]



from __future__ import annotations
from typing import Protocol, runtime_checkable
from zempy.zosapi.analysis.enums.show_as import ShowAsEnum
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes
from zempy.zosapi.analysis.ias.enums.rotations import Rotations
from zempy.zosapi.analysis.ias.enums.polarizations import Polarizations
from zempy.zosapi.analysis.ias.enums.star_effects_options import STAREffectsOptions

@runtime_checkable
class IAS_WavefrontMap(Protocol):

    @property
    def ShowAs(self) -> ShowAsEnum: ...
    @ShowAs.setter
    def ShowAs(self, v: ShowAsEnum) -> None: ...


    @property
    def Rotation(self) -> Rotations: ...
    @Rotation.setter
    def Rotation(self, v: Rotations) -> None: ...

    @property
    def SampleSize(self) -> SampleSizes: ...
    @SampleSize.setter
    def SampleSize(self, v: SampleSizes) -> None: ...

    @property
    def Polarization(self) -> Polarizations: ...
    @Polarization.setter
    def Polarization(self, v: Polarizations) -> None: ...

    @property
    def ReferenceToPrimary(self) -> bool: ...
    @ReferenceToPrimary.setter
    def ReferenceToPrimary(self, v: bool) -> None: ...

    @property
    def UseExitPupil(self) -> bool: ...
    @UseExitPupil.setter
    def UseExitPupil(self, v: bool) -> None: ...

    @property
    def RemoveTilt(self) -> bool: ...
    @RemoveTilt.setter
    def RemoveTilt(self, v: bool) -> None: ...

    @property
    def Scale(self) -> float: ...
    @Scale.setter
    def Scale(self, v: float) -> None: ...

    @property
    def Subaperture_X(self) -> float: ...
    @Subaperture_X.setter
    def Subaperture_X(self, v: float) -> None: ...

    @property
    def Subaperture_Y(self) -> float: ...
    @Subaperture_Y.setter
    def Subaperture_Y(self, v: float) -> None: ...

    @property
    def Subaperture_R(self) -> float: ...
    @Subaperture_R.setter
    def Subaperture_R(self, v: float) -> None: ...

    @property
    def ContourFormat(self) -> str: ...
    @ContourFormat.setter
    def ContourFormat(self, v: str) -> None: ...

    @property
    def STAREffects(self) -> STAREffectsOptions: ...
    @STAREffects.setter
    def STAREffects(self, v: STAREffectsOptions) -> None: ...



from zempy.zosapi.core.enum_base import ZosEnumBase


class PsfSpread(ZosEnumBase):
    Line = 0
    Edge = 1

PsfSpread._NATIVE_PATH = "ZOSAPI.Analysis.Settings.PsfSpread"
PsfSpread._ALIASES_EXTRA = {}

__all__ = ["PsfSpread"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class ReferenceGia(ZosEnumBase):
    ChiefRay = 0
    Vertex = 1
    PrimaryChief = 2
    Centroid = 3

ReferenceGia._NATIVE_PATH = "ZOSAPI.Analysis.Settings.ReferenceGia"
ReferenceGia._ALIASES_EXTRA = {}

__all__ = ["ReferenceGia"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class PsfTypes(ZosEnumBase):
    X_Linear = 0
    Y_Linear = 1
    X_Logarithmic = 2
    Y_Logarithmic = 3
    X_Phase = 4
    Y_Phase = 5
    X_RealPart = 6
    Y_RealPart = 7
    X_ImaginaryPart = 8
    Y_ImaginaryPart = 9

PsfTypes._NATIVE_PATH = "ZOSAPI.Analysis.Settings.PsfTypes"
PsfTypes._ALIASES_EXTRA = {}

__all__ = ["PsfTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class DisplayOption(ZosEnumBase):
    AllRays = 0
    FailedRays = 1
    PassedRays = 2

DisplayOption._NATIVE_PATH = "ZOSAPI.Analysis.Settings.DisplayOption"
DisplayOption._ALIASES_EXTRA = {}

__all__ = ["DisplayOption"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfaceSlopeCrossData(ZosEnumBase):
    TangentialSlope = 0
    SagittalSlope = 1
    XSlope = 2
    YSlope = 3
    SlopeModulus = 4
    SlopeUnused = 5

SurfaceSlopeCrossData._NATIVE_PATH = "ZOSAPI.Analysis.SurfaceSlopeCrossData"
SurfaceSlopeCrossData._ALIASES_EXTRA = {}

__all__ = ["SurfaceSlopeCrossData"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class AxisType(ZosEnumBase):
    X = 0
    Y = 1
    Z = 2

AxisType._NATIVE_PATH = "ZOSAPI.Analysis.Settings.AxisType"
AxisType._ALIASES_EXTRA = {}

__all__ = ["AxisType"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class HuygensSurfaceMftShowAsTypes(ZosEnumBase):
    GreyScale = 0
    InverseGreyScale = 1
    FalseColor = 2
    InverseFalseColor = 3

HuygensSurfaceMftShowAsTypes._NATIVE_PATH = "ZOSAPI.Analysis.HuygensSurfaceMftShowAsTypes"
HuygensSurfaceMftShowAsTypes._ALIASES_EXTRA = {}

__all__ = ["HuygensSurfaceMftShowAsTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class RemoveOptions(ZosEnumBase):
    None = 0
    BaseROC = 1
    BestFitSphere = 2
    BaseSag = 3
    CompositeSag = 4

RemoveOptions._NATIVE_PATH = "ZOSAPI.Analysis.RemoveOptions"
RemoveOptions._ALIASES_EXTRA = {}

__all__ = ["RemoveOptions"]



from __future__ import annotations
from typing import Protocol, runtime_checkable

from zempy.zosapi.editors.protocols.i_solve_data import ISolveData

@runtime_checkable
class ISolvePosition(ISolveData, Protocol):
    """Protocol for ISolvePosition (refactored to inherit ISolveData; unique fields only)."""
    @property
    def FromSurface(self) -> int: ...
    @FromSurface.setter
    def FromSurface(self, value: int) -> None: ...
    @property
    def Length(self) -> float: ...
    @Length.setter
    def Length(self, value: float) -> None: ...
    @property
    def Documentation(self) -> str: ...
    @property
    def set(self) -> str: ...
    @property
    def by(self) -> str: ...



from zempy.zosapi.core.enum_base import ZosEnumBase


class GridPlotType(ZosEnumBase):
    Surface = 0
    Contour = 1
    GrayScale = 2
    InverseGrayScale = 3
    FalseColor = 4
    InverseFalseColor = 5

GridPlotType._NATIVE_PATH = "ZOSAPI.Analysis.GridPlotType"
GridPlotType._ALIASES_EXTRA = {}

__all__ = ["GridPlotType"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class Rotations(ZosEnumBase):
    Rotate_0 = 0
    Rotate_90 = 1
    Rotate_180 = 2
    Rotate_270 = 3

Rotations._NATIVE_PATH = "ZOSAPI.Analysis.Settings.Rotations"
Rotations._ALIASES_EXTRA = {}

__all__ = ["Rotations"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SampleSizes_Pow2Plus1_X(ZosEnumBase):
    S_33x33 = 0
    S_65x65 = 1
    S_129x129 = 2
    S_257x257 = 3
    S_513x513 = 4
    S_1025x1025 = 5
    S_2049x2049 = 6
    S_4097x4097 = 7
    S_8193x8193 = 8

SampleSizes_Pow2Plus1_X._NATIVE_PATH = "ZOSAPI.Analysis.SampleSizes_Pow2Plus1_X"
SampleSizes_Pow2Plus1_X._ALIASES_EXTRA = {}

__all__ = ["SampleSizes_Pow2Plus1_X"]



from zempy.zosapi.analysis.fftpsf.adapter import fftpsf, fft_psf_settings
from zempy.zosapi.analysis.huygens_psf.adapters import huygens_psf_settings, huygens_psf
from zempy.zosapi.analysis.zernike_standard.adapters import zernike_standard_coefficients, zernike_standard_coefficients_settings




from __future__ import annotations
from typing import Protocol, runtime_checkable
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes

@runtime_checkable
class IAS_ZernikeStandardCoefficients(Protocol):
    """Protocol for ZOSAPI.Analysis.Settings.Aberrations.IAS_ZernikeStandardCoefficients."""

    @property
    def SampleSize(self) -> SampleSizes:
        ...

    @SampleSize.setter
    def SampleSize(self, v: SampleSizes) -> None:
        ...

    @property
    def ReferenceOBDToVertex(self) -> bool:
        ...

    @ReferenceOBDToVertex.setter
    def ReferenceOBDToVertex(self, v: bool) -> None:
        ...

    @property
    def Sx(self) -> float:
        ...

    @Sx.setter
    def Sx(self, v: float) -> None:
        ...

    @property
    def Sy(self) -> float:
        ...

    @Sy.setter
    def Sy(self, v: float) -> None:
        ...

    @property
    def Sr(self) -> float:
        ...

    @Sr.setter
    def Sr(self, v: float) -> None:
        ...

    @property
    def Epsilon(self) -> float:
        ...

    @Epsilon.setter
    def Epsilon(self, v: float) -> None:
        ...

    @property
    def MaximumNumberOfTerms(self) -> int:
        ...

    @MaximumNumberOfTerms.setter
    def MaximumNumberOfTerms(self, v: int) -> None:
        ...



from zempy.zosapi.core.enum_base import ZosEnumBase


class SampleSizes(ZosEnumBase):
    S_32x32 = 0
    S_64x64 = 1
    S_128x128 = 2
    S_256x256 = 3
    S_512x512 = 4
    S_1024x1024 = 5
    S_2048x2048 = 6
    S_4096x4096 = 7
    S_8192x8192 = 8
    S_16384x16384 = 9

SampleSizes._NATIVE_PATH = "ZOSAPI.Analysis.SampleSizes"
SampleSizes._ALIASES_EXTRA = {}

__all__ = ["SampleSizes"]



from __future__ import annotations
from typing import Any
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes
from zempy.zosapi.analysis.ias.adapters.ias import IAS
from zempy.zosapi.analysis.ias.adapters.registry import register_settings
from zempy.zosapi.analysis.enums.analysis_idm import AnalysisIDM
from zempy.zosapi.analysis.zernike_standard.protocols.ias_zernike_standard_coefficients import IAS_ZernikeStandardCoefficients
from zempy.zosapi.core.property_scalar import PropertyScalar
from zempy.zosapi.core.property_enum import property_enum
from zempy.zosapi.core.types_var import Z


@register_settings(AnalysisIDM.ZernikeStandardCoefficients)
class ZernikeStandardCoefficientsSettings(IAS[Z, IAS_ZernikeStandardCoefficients]):
    REQUIRED_NATIVE_ATTRS = IAS.REQUIRED_NATIVE_ATTRS + (
        "SampleSize",
        "ReferenceOBDToVertex",
        "Sx",
        "Sy",
        "Sr",
        "Epsilon",
        "MaximumNumberOfTerms",
    )

    SampleSize = property_enum("SampleSize", SampleSizes)
    ReferenceOBDToVertex = PropertyScalar(
        "ReferenceOBDToVertex", coerce_get=bool, coerce_set=bool
    )

    Sx = PropertyScalar("Sx", coerce_get=float, coerce_set=float)
    Sy = PropertyScalar("Sy", coerce_get=float, coerce_set=float)
    Sr = PropertyScalar("Sr", coerce_get=float, coerce_set=float)

    Epsilon = PropertyScalar("Epsilon", coerce_get=float, coerce_set=float)

    MaximumNumberOfTerms = PropertyScalar(
        "MaximumNumberOfTerms", coerce_get=int, coerce_set=int
    )

    # --- Native coercion hook (same pattern as FftPsfSettings) ---

    @staticmethod
    def _coerce_native(base: Any) -> Any:
        cast_fn = getattr(
            base,
            "As_ZOSAPI_Analysis_Settings_Aberrations_IAS_ZernikeStandardCoefficients",
            None,
        )
        if callable(cast_fn):
            obj = cast_fn()
            if obj is not None:
                return obj
        return getattr(base, "__implementation__", base)

    NATIVE_COERCER = _coerce_native

    def __str__(self) -> str:
        return (
            "Zernike Standard Coefficients settings:(\n"
            f"  field={self.Field},\n"
            f"  wavelength={self.Wavelength},\n"
            f"  surface={self.Surface},\n"
            f"  sample_size={self.SampleSize.name},\n"
            f"  reference_obd_to_vertex={self.ReferenceOBDToVertex},\n"
            f"  sx={self.Sx}, sy={self.Sy}, sr={self.Sr},\n"
            f"  epsilon={self.Epsilon},\n"
            f"  max_terms={self.MaximumNumberOfTerms}\n"
            ")"
        )



from zempy.zosapi.core.enum_base import ZosEnumBase


class SourceGia(ZosEnumBase):
    Uniform = 0
    Lambertian = 1

SourceGia._NATIVE_PATH = "ZOSAPI.Analysis.Settings.SourceGia"
SourceGia._ALIASES_EXTRA = {}

__all__ = ["SourceGia"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class Polarizations(ZosEnumBase):
    None = 0
    Ex = 1
    Ey = 2
    Ez = 3

Polarizations._NATIVE_PATH = "ZOSAPI.Analysis.Settings.Polarizations"
Polarizations._ALIASES_EXTRA = {}

__all__ = ["Polarizations"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfaceSlopeData(ZosEnumBase):
    SurfaceSlope = 0
    TangentialSlope = 1
    SagittalSlope = 2
    XSlope = 3
    YSlope = 4
    SlopeModulus = 5
    SlopeUnused = 6

SurfaceSlopeData._NATIVE_PATH = "ZOSAPI.Analysis.SurfaceSlopeData"
SurfaceSlopeData._ALIASES_EXTRA = {}

__all__ = ["SurfaceSlopeData"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class HuygensPsfTypes(ZosEnumBase):
    Linear = 0
    Log_Minus_1 = 1
    Log_Minus_2 = 2
    Log_Minus_3 = 3
    Log_Minus_4 = 4
    Log_Minus_5 = 5
    Real = 6
    Imaginary = 7
    Phase = 8

HuygensPsfTypes._NATIVE_PATH = "ZOSAPI.Analysis.Settings.HuygensPsfTypes"
HuygensPsfTypes._ALIASES_EXTRA = {}

__all__ = ["HuygensPsfTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfacePhaseData(ZosEnumBase):
    SurfacePhase = 0

SurfacePhaseData._NATIVE_PATH = "ZOSAPI.Analysis.SurfacePhaseData"
SurfacePhaseData._ALIASES_EXTRA = {}

__all__ = ["SurfacePhaseData"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class HuygensShowAsTypes(ZosEnumBase):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    TrueColor = 6

HuygensShowAsTypes._NATIVE_PATH = "ZOSAPI.Analysis.HuygensShowAsTypes"
HuygensShowAsTypes._ALIASES_EXTRA = {}

__all__ = ["HuygensShowAsTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class BestFitSphereOptions(ZosEnumBase):
    MinimumVolume = 0
    MinimumRMS = 1
    MinimumRMSWithOffset = 2

BestFitSphereOptions._NATIVE_PATH = "ZOSAPI.Analysis.BestFitSphereOptions"
BestFitSphereOptions._ALIASES_EXTRA = {}

__all__ = ["BestFitSphereOptions"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SampleSizes_Pow2Plus1(ZosEnumBase):
    S_33 = 0
    S_65 = 1
    S_129 = 2
    S_257 = 3
    S_513 = 4
    S_1025 = 5
    S_2049 = 6
    S_4097 = 7
    S_8193 = 8

SampleSizes_Pow2Plus1._NATIVE_PATH = "ZOSAPI.Analysis.SampleSizes_Pow2Plus1"
SampleSizes_Pow2Plus1._ALIASES_EXTRA = {}

__all__ = ["SampleSizes_Pow2Plus1"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfaceSagData(ZosEnumBase):
    SurfaceSag = 0

SurfaceSagData._NATIVE_PATH = "ZOSAPI.Analysis.SurfaceSagData"
SurfaceSagData._ALIASES_EXTRA = {}

__all__ = ["SurfaceSagData"]



from __future__ import annotations
from typing import Any
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes
from zempy.zosapi.analysis.ias.enums.huygens_psf_types import HuygensPsfTypes
from zempy.zosapi.analysis.enums.huygens_show_as_types import HuygensShowAsTypes
from zempy.zosapi.analysis.ias.enums.rotations import Rotations
from zempy.zosapi.analysis.ias.adapters.ias import IAS
from zempy.zosapi.analysis.ias.adapters.registry import register_settings
from zempy.zosapi.analysis.enums.analysis_idm import AnalysisIDM
from zempy.zosapi.analysis.huygens_psf.protocols.ias_huygens_psf import IAS_HuygensPsf
from zempy.zosapi.core.property_scalar import PropertyScalar
from zempy.zosapi.core.property_enum import property_enum
from zempy.zosapi.core.types_var import Z


@register_settings(AnalysisIDM.HuygensPsf)
class HuygensPsfSettings(IAS[Z, IAS_HuygensPsf]):
    #!!! Huygens doesnt have Surface as parameter
    REQUIRED_NATIVE_ATTRS = (
        "Field",
        "Wavelength",
        "PupilSampleSize",
        "ImageSampleSize",
        "Type",
        "ShowAsType",
        "Rotation",
        "Normalize",
        "UseCentroid",
        "UsePolarization",
        "ImageDelta",
        "Configuration",
    )


    PupilSampleSize = property_enum("PupilSampleSize", SampleSizes)
    ImageSampleSize = property_enum("ImageSampleSize", SampleSizes)
    Type            = property_enum("Type",            HuygensPsfTypes)
    ShowAsType      = property_enum("ShowAsType",      HuygensShowAsTypes)
    Rotation        = property_enum("Rotation",        Rotations)
    Normalize       = PropertyScalar("Normalize",       coerce_get=bool,  coerce_set=bool)
    UseCentroid     = PropertyScalar("UseCentroid",     coerce_get=bool,  coerce_set=bool)
    UsePolarization = PropertyScalar("UsePolarization", coerce_get=bool,  coerce_set=bool)
    ImageDelta      = PropertyScalar("ImageDelta",      coerce_get=float, coerce_set=float)
    Configuration   = PropertyScalar("Configuration",   coerce_get=int,   coerce_set=int)

    # --- Native coercion hook (same pattern as PSFSettings) ---
    @staticmethod
    def _coerce_native(base: Any) -> Any:
        # Try the typed cast helper generated by ZOSAPI:
        #   As_ZOSAPI_Analysis_Settings_Psf_IAS_HuygensPsf
        cast_fn = getattr(base, "As_ZOSAPI_Analysis_Settings_Psf_IAS_HuygensPsf", None)
        if callable(cast_fn):
            obj = cast_fn()
            if obj is not None:
                return obj
        # Fallback: for some wrappers, the real .NET object is under __implementation__
        return getattr(base, "__implementation__", base)

    NATIVE_COERCER = _coerce_native

    def __str__(self) -> str:
        return (
            "Huygens PSF settings:(\n"
            f"  field={self.Field},\n"
            f"  wavelength={self.Wavelength},\n"            
            f"  pupil_sample_size={self.PupilSampleSize.name},\n"
            f"  image_sample_size={self.ImageSampleSize.name},\n"
            f"  type={self.Type.name},\n"
            f"  show_as={self.ShowAsType.name},\n"
            f"  rotation={self.Rotation.name},\n"
            f"  normalize={self.Normalize},\n"
            f"  use_centroid={self.UseCentroid},\n"
            f"  use_polarization={self.UsePolarization},\n"
            f"  image_delta={self.ImageDelta},\n"
            f"  configuration={self.Configuration}\n"
            ")"
        )



from zempy.zosapi.core.enum_base import ZosEnumBase


class DetectorViewerScaleTypes(ZosEnumBase):
    Linear = 0
    Log_Minus_5 = 1
    Normalized = 2
    Log_Minus_10 = 3
    Log_Minus_15 = 4

DetectorViewerScaleTypes._NATIVE_PATH = "ZOSAPI.Analysis.Settings.DetectorViewerScaleTypes"
DetectorViewerScaleTypes._ALIASES_EXTRA = {}

__all__ = ["DetectorViewerScaleTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class DetectorViewerShowDataTypes(ZosEnumBase):
    IncidentFlux = 0
    AbsorbedFlux = 1
    AbsorbedFluxVolume = 2
    PositionSpace = 3
    AngleSpace = 4
    Polar_AngleSpace = 5
    IncoherentIrradiance = 6
    CoherentIrradiance = 7
    CoherentPhase = 8
    RadiantIntensity = 9
    RadiancePositionSpace = 10
    RadianceAngleSpace = 11
    IncoherentIlluminance = 12
    CoherentIlluminance = 13
    LuminousIntensity = 14
    LuminancePositionSpace = 15
    LuminanceAngleSpace = 16
    IncoherentFluence = 17
    CoherentFluence = 18

DetectorViewerShowDataTypes._NATIVE_PATH = "ZOSAPI.Analysis.DetectorViewerShowDataTypes"
DetectorViewerShowDataTypes._ALIASES_EXTRA = {}

__all__ = ["DetectorViewerShowDataTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class DetectorViewerShowAsTypes(ZosEnumBase):
    FullListing = 0
    AzimuthCrossSection = 1
    Text_CrossSection_Row = 2
    Text_CrossSection_Column = 3
    FluxVsWaveLength = 4
    GreyScale = 5
    GreyScale_Inverted = 6
    FalseColor = 7
    FalseColor_Inverted = 8
    TrueColor = 9
    Color_CrossSection_Row = 10
    Color_CrossSection_Column = 11
    Color_FluxVsWavelength = 12
    CrossSection = 13
    Directivity_Full = 14
    Directivity_Half = 15
    CrossSection_Row = 16
    CrossSection_Column = 17
    GeometricMtf = 18

DetectorViewerShowAsTypes._NATIVE_PATH = "ZOSAPI.Analysis.DetectorViewerShowAsTypes"
DetectorViewerShowAsTypes._ALIASES_EXTRA = {}

__all__ = ["DetectorViewerShowAsTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfacePhaseSlopeData(ZosEnumBase):
    PhaseSlopeTangential = 0
    PhaseSlopeSagittal = 1
    PhaseSlopeX = 2
    PhaseSlopeY = 3
    PhaseSlopeModulus = 4
    PhaseSlopeUnused = 5

SurfacePhaseSlopeData._NATIVE_PATH = "ZOSAPI.Analysis.SurfacePhaseSlopeData"
SurfacePhaseSlopeData._ALIASES_EXTRA = {}

__all__ = ["SurfacePhaseSlopeData"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class STAREffectsOptions(ZosEnumBase):
    On = 0
    Difference = 1

STAREffectsOptions._NATIVE_PATH = "ZOSAPI.Analysis.Settings.STAREffectsOptions"
STAREffectsOptions._ALIASES_EXTRA = {}

__all__ = ["STAREffectsOptions"]



from __future__ import annotations
from typing import Any

from zempy.zosapi.analysis.enums.show_as import ShowAsEnum
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes
from zempy.zosapi.analysis.ias.enums.rotations import Rotations
from zempy.zosapi.analysis.ias.enums.polarizations import Polarizations
from zempy.zosapi.analysis.ias.enums.star_effects_options import STAREffectsOptions

from zempy.zosapi.analysis.ias.adapters.ias import IAS
from zempy.zosapi.analysis.ias.adapters.registry import register_settings
from zempy.zosapi.analysis.enums.analysis_idm import AnalysisIDM

from zempy.zosapi.analysis.wavefront_map.protocols.ias_wavefront_map import IAS_WavefrontMap

from zempy.zosapi.core.property_scalar import PropertyScalar
from zempy.zosapi.core.property_enum import property_enum
from zempy.zosapi.core.types_var import Z


@register_settings(AnalysisIDM.WavefrontMap)
class WavefrontMapSettings(IAS[Z, IAS_WavefrontMap]):
    """
    Adapter for ZOSAPI.Analysis.Settings.IAS_WavefrontMap
    following the same pattern as HuygensPsfSettings.
    """

    REQUIRED_NATIVE_ATTRS = (
        "Field",
        "Surface",
        "Wavelength",
        "ShowAs",
        "Rotation",
        "SampleSize",
        "Polarization",
        "ReferenceToPrimary",
        "UseExitPupil",
        "RemoveTilt",
        "Scale",
        "Subaperture_X",
        "Subaperture_Y",
        "Subaperture_R",
        "ContourFormat",
        "STAREffects",
    )

    # --- Enums ---
    ShowAs       = property_enum("ShowAs",       ShowAsEnum)
    Rotation     = property_enum("Rotation",     Rotations)
    SampleSize   = property_enum("SampleSize",   SampleSizes)
    Polarization = property_enum("Polarization", Polarizations)
    STAREffects  = property_enum("STAREffects",  STAREffectsOptions)

    # --- Scalars / flags ---
    ReferenceToPrimary = PropertyScalar(
        "ReferenceToPrimary", coerce_get=bool, coerce_set=bool
    )
    UseExitPupil = PropertyScalar(
        "UseExitPupil", coerce_get=bool, coerce_set=bool
    )
    RemoveTilt = PropertyScalar(
        "RemoveTilt", coerce_get=bool, coerce_set=bool
    )
    Scale = PropertyScalar(
        "Scale", coerce_get=float, coerce_set=float
    )

    Subaperture_X = PropertyScalar(
        "Subaperture_X", coerce_get=float, coerce_set=float
    )
    Subaperture_Y = PropertyScalar(
        "Subaperture_Y", coerce_get=float, coerce_set=float
    )
    Subaperture_R = PropertyScalar(
        "Subaperture_R", coerce_get=float, coerce_set=float
    )

    ContourFormat = PropertyScalar(
        "ContourFormat", coerce_get=str, coerce_set=str
    )

    # --- Native coercion hook (same pattern as HuygensPsfSettings) ---
    @staticmethod
    def _coerce_native(base: Any) -> Any:
        # Likely cast helper from ZOSAPI:
        #   As_ZOSAPI_Analysis_Settings_Wavefront_IAS_WavefrontMap
        cast_fn = getattr(
            base,
            "As_ZOSAPI_Analysis_Settings_Wavefront_IAS_WavefrontMap",
            None,
        )
        if callable(cast_fn):
            obj = cast_fn()
            if obj is not None:
                return obj
        # Fallback: some wrappers store the real .NET object under __implementation__
        return getattr(base, "__implementation__", base)

    NATIVE_COERCER = _coerce_native

    def __str__(self) -> str:
        return (
            "Wavefront Map settings:(\n"
            f"  field={self.Field},\n"
            f"  surface={self.Surface},\n"
            f"  wavelength={self.Wavelength},\n"
            f"  show_as={self.ShowAs.name},\n"
            f"  rotation={self.Rotation.name},\n"
            f"  sample_size={self.SampleSize.name},\n"
            f"  polarization={self.Polarization.name},\n"
            f"  reference_to_primary={self.ReferenceToPrimary},\n"
            f"  use_exit_pupil={self.UseExitPupil},\n"
            f"  remove_tilt={self.RemoveTilt},\n"
            f"  scale={self.Scale},\n"
            f"  subaperture_x={self.Subaperture_X},\n"
            f"  subaperture_y={self.Subaperture_Y},\n"
            f"  subaperture_r={self.Subaperture_R},\n"
            f"  contour_format={self.ContourFormat},\n"
            f"  star_effects={self.STAREffects.name},\n"
            ")"
        )



from zempy.zosapi.core.enum_base import ZosEnumBase


class UserAnalysisDataType(ZosEnumBase):
    None = 0
    Line2D = 1
    Grid = 2
    GridRGB = 3
    Text = 4

UserAnalysisDataType._NATIVE_PATH = "ZOSAPI.Analysis.UserAnalysisDataType"
UserAnalysisDataType._ALIASES_EXTRA = {}

__all__ = ["UserAnalysisDataType"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class SurfaceCurvatureCrossData(ZosEnumBase):
    TangentialCurvature = 0
    SagittalCurvature = 1
    X_Curvature = 2
    Y_Curvature = 3
    CurvatureModulus = 4
    CurvatureUnused = 5

SurfaceCurvatureCrossData._NATIVE_PATH = "ZOSAPI.Analysis.SurfaceCurvatureCrossData"
SurfaceCurvatureCrossData._ALIASES_EXTRA = {}

__all__ = ["SurfaceCurvatureCrossData"]



from __future__ import annotations
from typing import  cast, TYPE_CHECKING
from zempy.zosapi.core.types_var import N, Z
from zempy.zosapi.analysis.adapters.ia import IA
from zempy.zosapi.analysis.enums.analysis_idm import AnalysisIDM
from zempy.zosapi.analysis.adapters.analysis_registry import register_analysis
from zempy.zosapi.analysis.huygens_psf.adapters.huygens_psf_settings import HuygensPsfSettings

if TYPE_CHECKING:
    from zempy.zosapi.analysis.huygens_psf.protocols.ias_huygens_psf import IAS_HuygensPsf

@register_analysis(AnalysisIDM.HuygensPsf)
class HuygensPSF(IA[HuygensPsfSettings, Z, N]):

    def __str__(self) -> str:
        return f"FFT PSF settings={self.settings}"

    @property
    def settings(self) -> IAS_HuygensPsf:
        return cast(IAS_HuygensPsf, super().settings)


from zempy.zosapi.core.enum_base import ZosEnumBase


class ScanTypes(ZosEnumBase):
    Plus_Y = 0
    Plus_X = 1
    Minus_Y = 2
    Minus_X = 3

ScanTypes._NATIVE_PATH = "ZOSAPI.Analysis.Settings.ScanTypes"
ScanTypes._ALIASES_EXTRA = {}

__all__ = ["ScanTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class GiaShowAsTypes(ZosEnumBase):
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    SpotDiagram = 6
    CrossX = 7
    CrossY = 8

GiaShowAsTypes._NATIVE_PATH = "ZOSAPI.Analysis.GiaShowAsTypes"
GiaShowAsTypes._ALIASES_EXTRA = {}

__all__ = ["GiaShowAsTypes"]



from zempy.zosapi.core.enum_base import ZosEnumBase


class Parity(ZosEnumBase):
    Even = 0
    Odd = 1

Parity._NATIVE_PATH = "ZOSAPI.Analysis.Settings.Parity"
Parity._ALIASES_EXTRA = {}

__all__ = ["Parity"]



from __future__ import annotations
from typing import Protocol, runtime_checkable
from zempy.zosapi.analysis.enums.sample_sizes import SampleSizes
from zempy.zosapi.analysis.ias.enums.huygens_psf_types import HuygensPsfTypes
from zempy.zosapi.analysis.enums.huygens_show_as_types import HuygensShowAsTypes
from zempy.zosapi.analysis.ias.enums.rotations import Rotations

@runtime_checkable
class IAS_HuygensPsf(Protocol):


    @property
    def PupilSampleSize(self) -> SampleSizes: ...
    @PupilSampleSize.setter
    def PupilSampleSize(self, v: SampleSizes) -> None: ...

    @property
    def ImageSampleSize(self) -> SampleSizes: ...
    @ImageSampleSize.setter
    def ImageSampleSize(self, v: SampleSizes) -> None: ...

    @property
    def Type(self) -> HuygensPsfTypes: ...
    @Type.setter
    def Type(self, v: HuygensPsfTypes) -> None: ...

    @property
    def ShowAsType(self) -> HuygensShowAsTypes: ...
    @ShowAsType.setter
    def ShowAsType(self, v: HuygensShowAsTypes) -> None: ...

    @property
    def Rotation(self) -> Rotations: ...
    @Rotation.setter
    def Rotation(self, v: Rotations) -> None: ...

    @property
    def Normalize(self) -> bool: ...
    @Normalize.setter
    def Normalize(self, v: bool) -> None: ...

    @property
    def UseCentroid(self) -> bool: ...
    @UseCentroid.setter
    def UseCentroid(self, v: bool) -> None: ...

    @property
    def UsePolarization(self) -> bool: ...
    @UsePolarization.setter
    def UsePolarization(self, v: bool) -> None: ...

    @property
    def ImageDelta(self) -> float: ...
    @ImageDelta.setter
    def ImageDelta(self, v: float) -> None: ...

    @property
    def Configuration(self) -> int: ...
    @Configuration.setter
    def Configuration(self, v: int) -> None: ...



from __future__ import annotations
from typing import  cast, TYPE_CHECKING
from zempy.zosapi.core.types_var import N, Z
from zempy.zosapi.analysis.adapters.ia import IA
from zempy.zosapi.analysis.enums.analysis_idm import AnalysisIDM
from zempy.zosapi.analysis.adapters.analysis_registry import register_analysis
from zempy.zosapi.analysis.zernike_standard.adapters.zernike_standard_coefficients_settings import ZernikeStandardCoefficientsSettings

if TYPE_CHECKING:
    from zempy.zosapi.analysis.zernike_standard.protocols.ias_zernike_standard_coefficients import IAS_ZernikeStandardCoefficients

@register_analysis(AnalysisIDM.ZernikeStandardCoefficients)
class ZernikeStandardCoefficients(IA[ZernikeStandardCoefficientsSettings, Z, N]):

    def __str__(self) -> str:
        return f"FFT PSF settings={self.settings}"

    @property
    def settings(self) -> IAS_ZernikeStandardCoefficients:
        return cast(IAS_ZernikeStandardCoefficients, super().settings)